package com.alivc.rtc.linux;

public abstract class AliRTCLinuxEngine {

    public enum ERROR_CODE {
        ERR_OK(0),

        /////// 加入频道错误码 //////////
        ERR_JOIN_BAD_APPID(0x02010201), //AppId不存在

        ERR_JOIN_INVALID_APPID(0x02010202), //AppId已失效

        ERR_JOIN_BAD_CHANNEL(0x02010204), //频道不存在

        ERR_JOIN_INVALID_CHANNEL(0x02010203), //频道已失效

        ERR_JOIN_BAD_TOKEN(0x02010205), //Token不存在

        ERR_JOIN_TIMEOUT(0x01020204), //加入频道超时

        ERR_JOIN_BAD_PARAM(0x01030101), //加入频道参数错误

        ///////// 媒体错误码 ////////////
        ERR_AUDIO_DATA_ERROR(0x03010201), //音频数据错误

        ERR_VIDEO_DATA_ERROR(0x03010202), //视频数据错误

        ///////// 网络错误码 ////////////
        ERR_NETWORK_CONNECT_FAIL(0x01050201), //媒体通道建立失败

        ERR_NETWORK_RECONNECT_FAIL(0x01050202), //媒体通道重连失败

        ERR_NETWORK_DISCONNECT(0x01050203), //连接断开

        ERR_NETWORK_TIMEOUT(0x0102020C), //连接超时

        ///////// 其他错误码 ////////////
        ERR_SDK_INVALID_STATE(0x01060101), //SDK内部状态错误

        ERR_SESSION_REMOVED(0x01060102), //session已经被移除

        ERR_INNER(0x01060103), //SDK内部错误

        ERR_VIDEO_TRANSFER(0x01060104), //视频数据传输错误

        ERR_AUDIO_TRANSFER(0x01060105), //音频数据传输错误

        ERR_AUDIO_BUFFER_FULL(0x01060106), //音频外部输入时，频率过快

        ERR_FATAL(-1);         //SDK严重错误

        private int value;

        ERROR_CODE(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum WARNNING_CODE {
        WARN_DEFAULT(-1);

        private int value;

        WARNNING_CODE(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum ChannelProfile {
        ChannelProfileCommunication(0),                      ///< 通信模式
        ChannelProfileInteractiveLive(1);                    ///< 直播模式

        private int value;

        ChannelProfile(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static ChannelProfile getByValue(int value) {
            for (ChannelProfile e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public enum ClientRole {
        ClientRoleInteractive(0),                        ///< 互动角色
        ClientRoleLive(1);                     ///< 观众角色

        private int value;

        ClientRole(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static ClientRole getByValue(int value) {
            for (ClientRole e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public enum PublishMode {
        PublishAutomatically(0),
        PublishManually(1);

        private int value;

        PublishMode(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum VideoTrack {
        VideoTrackNo(0),  ///< no video track
        VideoTrackCamera(1),  ///< video from camera, file, etc.
        VideoTrackScreen(2),  ///< video from screen sharing
        VideoTrackBoth(3),  ///< both VideoTrackCamera and VideoTrackScreen
        VideoTrackEnd(0xffffffff);

        private int value;

        VideoTrack(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static VideoTrack getByValue(int value) {
            for (VideoTrack e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }

    }

    public enum AudioTrack {
        AudioTrackNo(0),  ///< no audio track
        AudioTrackMic(1),  ///< audio from mic, file, etc.
        AudioTrackEnd(0xffffffff);
        private int value;

        AudioTrack(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum VideoProfile {
        VideoProfile_Default(0), // let sdk decide
        VideoProfile_180_240P_15(1),
        VideoProfile_180_320P_15(2),
        VideoProfile_180_320P_30(3),
        VideoProfile_240_320P_15(4),
        VideoProfile_360_480P_15(5),
        VideoProfile_360_480P_30(6),
        VideoProfile_360_640P_15(7),
        VideoProfile_360_640P_30(8),
        VideoProfile_480_640P_15(9),
        VideoProfile_480_640P_30(10),
        VideoProfile_720_960P_15(11),
        VideoProfile_720_960P_30(12),
        VideoProfile_720_1280P_15(13),
        VideoProfile_720_1280P_30(14),
        VideoProfile_1080_1920P_15(15),
        VideoProfile_1080_1920P_30(16),
        VideoProfile_480_640P_15_1500Kb(17);

        private int value;

        VideoProfile(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static VideoProfile getByValue(int value) {
            for (VideoProfile e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public enum VideoSource {
        VideoSourceCamera(0),
        VideoSourceScreenShare(1),
        VideosourceTypeMax(0xffffffff);

        private int value;

        VideoSource(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static VideoSource getByValue(int value) {
            for (VideoSource e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public enum RenderMode {
        RenderModeAuto(0),    //自动模式
        RenderModeStretch(1),    //延伸
        RenderModeFill(2),    //填充
        RenderModeCrop(3);    //裁剪
        private int value;

        RenderMode(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static RenderMode getByValue(int value) {
            for (RenderMode e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public enum VideoDataFormat {
        VideoDataFormatUnknow(-1),
        VideoDataFormatBGRA(0),
        VideoDataFormatI420(1),
        VideoDataFormatNV21(2),
        VideoDataFormatNV12(3);

        private int value;

        VideoDataFormat(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum VideoBufferType {
        VideoBufferTypeRawData(0),
        VideoBufferTypeCVPixelBuffer(1),
        VideoBufferTypeTexture(2);

        private int value;

        VideoBufferType(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public static class VideoDataSample {
        public VideoDataFormat format;
        public VideoBufferType bufferType;
        public byte[] data;
        public int dataLen;
        public int strideY;
        public int strideU;
        public int strideV;
        public int height;
        public int width;
        public int rotation;
        public long timeStamp;
    }


    /**
     * The Following Define Only Design For Recording
     */
    public enum RecordingMode {
        RecordingAutomatically(0),
        RecordingManually(1);

        private int value;

        RecordingMode(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum RecordingVideoTrackType {
        RecordingVideoTrackCameraLarge(0),
        RecordingVideoTrackCameraSmall(1),
        RecordingVideoTrackCameraSuper(2),
        RecordingVideoTrackScreenshare(3);

        private int value;

        RecordingVideoTrackType(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static RecordingVideoTrackType getByValue(int value) {
            for (RecordingVideoTrackType e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public enum VideoFormat {
        VideoFormatH264(0);
        private int value;

        VideoFormat(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum AudioFormat {
        AudioFormatMixedPcm(0);
        private int value;

        AudioFormat(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum VideoFrameType {
        VideoFrameH264(0);

        private int value;

        VideoFrameType(int v) {
            this.value = v;
        }

        public int getValue() {
            return value;
        }

        public static VideoFrameType getByValue(int value) {
            for (VideoFrameType e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public enum AudioFrameType {
        AudioFrameRawPcm(0);

        private int value;

        AudioFrameType(int v) {
            this.value = v;
        }

        public int getValue() {
            return value;
        }

        public static AudioFrameType getByValue(int value) {
            for (AudioFrameType e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    public static class AudioPcmFrame {
        /**
         * Timestamp
         */
        public long frame_ms;
        public int channels; // 1
        public int sample_bits; // 16
        public int sample_rates; // 8k, 16k, 32k
        public int samples;

        public byte[] pcmBuf;
        public int pcmBufSize;

        AudioPcmFrame(long frame_ms, int sample_rates, int samples) {
            this.frame_ms = frame_ms;
            this.sample_rates = sample_rates;
            this.samples = samples;
        }
    }

    public static class AudioFrame {
        public AudioFrameType type;
        public AudioPcmFrame pcm;
    }

    public static class VideoH264Frame {
        public long frame_ms;
        public int frame_num;
        public byte[] buf;
        public int bufSize;
    }

    public static class VideoFrame {
        public RecordingVideoTrackType track;
        public VideoFrameType type;
        public VideoH264Frame h264;
    }

    public static class JoinChannelConfig {
        public boolean isAudioOnly;
        public boolean isVideoOnly;
        public ChannelProfile channelProfile = ChannelProfile.ChannelProfileCommunication;
        public ClientRole clientRole = ClientRole.ClientRoleInteractive;
        public PublishMode publishMode = PublishMode.PublishAutomatically;
        public RecordingMode recordingMode = RecordingMode.RecordingAutomatically;
        public VideoFormat recordingVideoFormat = VideoFormat.VideoFormatH264;
        public AudioFormat recordingAudioFormat = AudioFormat.AudioFormatMixedPcm;
    }

    public static class AuthInfo {
        public String channel;
        public String userid;
        public String username;
        public String appid;
        public String nonce;
        public String token;
        public String[] gslb;
        public String[] agent;
        public int gslb_count;
        public int agent_count;
        public long timestamp;
    }

    public static AliRTCLinuxEngine createInstance(AliRTCLinuxEngineListener listener, int lowPort, int highPort, String logPath, String coreServicePath, boolean h5mode) {
        return new AliRTCLinuxEngineImpl(listener, lowPort, highPort, logPath, coreServicePath, h5mode);
    }

    public abstract void destroy();

    /**
     * @param authInfo 认证信息，从App Server获取。
     * @param config   joinChannel时的设置项
     * @brief 加入频道
     */
    public abstract int joinChannel(AuthInfo authInfo, JoinChannelConfig config);

    /**
     * @brief 离开频道
     */
    public abstract int leaveChannel();

    /**
     * @param profile 预定义的视频分辨率和帧率，详见VideoProfile
     * @param track   视频流的类型，详见VideoTrack
     * @brief 设置视频track的profile
     * @note 设置之后等到下次publish的时候才能生效
     */
    public abstract void setVideoProfile(VideoProfile profile, VideoTrack track);

    /**
     * @param track 视频流的类型，详见VideoTrack
     * @return 返回的是正在使用的（已经推流中）或者即将被使用的（下一次推流才会生效）视频分辨率和帧率，详见VideoProfile
     * @brief 获取视频track的profile
     * @note 返回值不一定是正在使用的video profile。另外VideoTrackScreen是不支持的
     */
    public abstract int getVideoProfile(VideoTrack track);

    /**
     * @param enable 是否允许
     * @brief 设置是否允许推送camera track
     * @note 需要调用publish才能生效
     * 缺省允许camera track推流
     */
    public abstract void configLocalCameraPublish(boolean enable);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送camera track
     */
    public abstract boolean isLocalCameraPublishEnabled();

    /**
     * @param enable 是否允许
     * @brief 设置是否允许推送screen track
     * @note 需要调用publish才能生效
     * 缺省不允许screen track推流
     */
    public abstract void configLocalScreenPublish(boolean enable);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送screen track
     */
    public abstract boolean isLocalScreenPublishEnabled();

    /**
     * @param enable 是否允许
     * @brief 设置是否允许推送audio
     * @note 需要调用publish才能生效
     * 缺省允许音频推流
     */
    public abstract void configLocalAudioPublish(boolean enable);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送audio track
     */
    public abstract boolean isLocalAudioPublishEnabled();

    /**
     * @param enabled true表示需要推送次流；false表示不推送次流
     * @param track   must be VideoTrackCamera
     * @brief 是否允许推送次要视频流
     * @note 需要调用publish才能生效
     * 缺省enable。目前只支持camera track，不支持screen track
     */
    public abstract int configLocalSimulcast(boolean enabled, VideoTrack track);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送simulcast (camera track)
     */
    public abstract boolean isLocalSimulcastEnabled();

    /**
     * @param enable     true 开启, false 关闭
     * @param useTexture 是否使用texture 模式，目前只支持false
     * @param source     流类型
     * @param renderMode 绘制模式
     * @brief 启用外部视频输入源
     * @note 启用后使用PushExternalVideoFrame接口输入视频数据
     */
    public abstract int setExternalVideoSource(boolean enable, boolean useTexture, VideoSource source, RenderMode renderMode);

    /**
     * @param frame  帧数据
     * @param source 流类型
     * @brief 输入外部输视频
     * @note 目前输入视频类型只支持I420
     */
    public abstract int pushExternalVideoFrame(VideoDataSample frame, VideoSource source);

    /**
     * @param enable           true 开启，false 关闭
     * @param sampleRate       采样率 16k 48k...
     * @param channelsPerFrame 采样率 16k 48k...
     * @return >=0表示成功， <0表示失败
     * @brief 设置是否启用外部音频输入推流
     * @note 可通过SetExternalAudioPublishVolume设置输入音频推流音量
     */
    public abstract int setExternalAudioSource(boolean enable, int sampleRate, int channelsPerFrame);

    /**
     * @param audioSamples 音频数据
     * @param sampleLength 音频数据长度
     * @param timestamp    时间戳
     * @return <0表示失败，返回值为ERR_AUDIO_BUFFER_FULL时，需要在间隔投递数据时间长度后再次重试投递
     * @brief 输入外部音频数据推流
     */
    public abstract int pushExternalAudioFrameRawData(byte[] audioSamples, int sampleLength, long timestamp);

    /**
     * @param volume 音量 0-100
     * @brief 设置外部输入音频推流混音音量
     */
    public abstract int setExternalAudioPublishVolume(int volume);

    /**
     * @return vol 音量
     * @brief 获取外部输入音频推流混音音量
     */
    public abstract int getExternalAudioPublishVolume();

    /**
     * @brief 手动推送视频和音频流
     * @note 需要推送的流通过API ConfigLocalAudioPublish, ConfigLocalCameraPublish和ConfigLocalScreenPublish进行设置
     * 如果需要手工配置所有推流和拉流，请通过JoinChannelConfig字段，再JoinChannel时，选择PublishManually
     * 推流结果异步返回，执行结果会通过onResult中返回
     */
    public abstract int publish();

    /**
     * @brief 手动开启录制
     * @note 如果需要手工配置所有推流和拉流，请通过JoinChannelConfig字段，再JoinChannel时，选择RecordingManually
     */
    public abstract int startRecording();

    /**
     * @brief 手动停止录制
     */
    public abstract int stopRecording();

    /**
     * @brief 创建媒体播放器
     */
    public abstract AliRTCLinuxMediaPlayer createMediaPlayer();

    /**
     * @brief 销毁媒体播放器
     */
    public abstract int destroyMediaPlayer(AliRTCLinuxMediaPlayer player);

}
