package com.alivc.rtc.linux;

class AliRTCLinuxNative {

    /********** engine **********/
    native long createAliRTCEngineInstance(Object listener, int lowPort, int highPort, String logPath, String coreServicePath, boolean h5mode);

    native void destroy(long handle);

    /**
     * @param authInfo 认证信息，从App Server获取。
     * @brief 加入频道
     */
    native int joinChannel(long handle, AliRTCLinuxEngine.AuthInfo authInfo, boolean isAudioOnly, boolean isVideoOnly, int channelProfile, int chientRole,
                           int publishMode, int recordingMode, int videoFormat, int audioFormat);

    /**
     * @brief 离开频道
     */
    native int leaveChannel(long handle);

    /**
     * @param profile 预定义的视频分辨率和帧率，详见VideoProfile
     * @param track   视频流的类型，详见VideoTrack
     * @brief 设置视频track的profile
     * @note 设置之后等到下次publish的时候才能生效
     */
    native void setVideoProfile(long handle, int profile, int track);

    /**
     * @param track 视频流的类型，详见VideoTrack
     * @return 返回的是正在使用的（已经推流中）或者即将被使用的（下一次推流才会生效）视频分辨率和帧率，详见VideoProfile
     * @brief 获取视频track的profile
     * @note 返回值不一定是正在使用的video profile。另外VideoTrackScreen是不支持的
     */
    native int getVideoProfile(long handle, int track);

    /**
     * @param enable 是否允许
     * @brief 设置是否允许推送camera track
     * @note 需要调用publish才能生效
     * 缺省允许camera track推流
     */
    native void configLocalCameraPublish(long handle, boolean enable);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送camera track
     */
    native boolean isLocalCameraPublishEnabled(long handle);

    /**
     * @param enable 是否允许
     * @brief 设置是否允许推送screen track
     * @note 需要调用publish才能生效
     * 缺省不允许screen track推流
     */
    native void configLocalScreenPublish(long handle, boolean enable);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送screen track
     */
    native boolean isLocalScreenPublishEnabled(long handle);

    /**
     * @param enable 是否允许
     * @brief 设置是否允许推送audio
     * @note 需要调用publish才能生效
     * 缺省允许音频推流
     */
    native void configLocalAudioPublish(long handle, boolean enable);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送audio track
     */
    native boolean isLocalAudioPublishEnabled(long handle);

    /**
     * @param enabled true表示需要推送次流；false表示不推送次流
     * @param track   must be VideoTrackCamera
     * @brief 是否允许推送次要视频流
     * @note 需要调用publish才能生效
     * 缺省enable。目前只支持camera track，不支持screen track
     */
    native int configLocalSimulcast(long handle, boolean enabled, int track);

    /**
     * @return true: 允许；false: 禁止
     * @brief 查询是否允许推送simulcast (camera track)
     */
    native boolean isLocalSimulcastEnabled(long handle);

    /**
     * @param enable     true 开启, false 关闭
     * @param useTexture 是否使用texture 模式，目前只支持false
     * @param source     流类型
     * @brief 启用外部视频输入源
     * @note 启用后使用PushExternalVideoFrame接口输入视频数据
     */
    native int setExternalVideoSource(long handle, boolean enable, boolean useTexture, int source, int renderMode);

    /**
     * @param source 流类型
     * @brief 输入外部输视频
     * @note 目前输入视频类型只支持I420
     */
    native int pushExternalVideoFrame(long handle, int format, int bufferType, byte[] data, int dataLen, int strideY, int strideU, int strideV, int width, int height, int rotation, long timestamp, int source);

    /**
     * @param enable           true 开启，false 关闭
     * @param sampleRate       采样率 16k 48k...
     * @param channelsPerFrame 采样率 16k 48k...
     * @return >=0表示成功， <0表示失败
     * @brief 设置是否启用外部音频输入推流
     * @note 可通过SetExternalAudioPublishVolume设置输入音频推流音量
     */
    native int setExternalAudioSource(long handle, boolean enable, int sampleRate, int channelsPerFrame);

    /**
     * @param audioSamples 音频数据
     * @param sampleLength 音频数据长度
     * @param timestamp    时间戳
     * @return <0表示失败，返回值为ERR_AUDIO_BUFFER_FULL时，需要在间隔投递数据时间长度后再次重试投递
     * @brief 输入外部音频数据推流
     */
    native int pushExternalAudioFrameRawData(long handle, byte[] audioSamples, int sampleLength, long timestamp);

    /**
     * @param volume 音量 0-100
     * @brief 设置外部输入音频推流混音音量
     */
    native int setExternalAudioPublishVolume(long handle, int volume);

    /**
     * @return vol 音量
     * @brief 获取外部输入音频推流混音音量
     */
    native int getExternalAudioPublishVolume(long handle);

    /**
     * @brief 手动推送视频和音频流
     * @note 需要推送的流通过API ConfigLocalAudioPublish, ConfigLocalCameraPublish和ConfigLocalScreenPublish进行设置
     * 如果需要手工配置所有推流和拉流，请通过JoinChannelConfig字段，再JoinChannel时，选择PublishManually
     * 推流结果异步返回，执行结果会通过onResult中返回
     */
    native int publish(long handle);

    /**
     * @brief 手动开启录制
     * @note 如果需要手工配置所有推流和拉流，请通过JoinChannelConfig字段，再JoinChannel时，选择RecordingManually
     */
    native int startRecording(long handle);

    /**
     * @brief 手动停止录制
     */
    native int stopRecording(long handle);

    /**
     * @brief 创建媒体播放器
     */
    native long createMediaPlayer(long handle);

    /**
     * @brief 销毁媒体播放器
     */
    native int destroyMediaPlayer(long handle, long player);

    /********** player **********/

    native int setEventHandler(long handle, Object eventHandler);

    native int loadResource(long handle, String path);

    native int start(long handle);

    native int stop(long handle);

    native int pause(long handle);

    native int resume(long handle);

    native int seekTo(long handle, long millisecond);

    native int setVideoSource(long handle, boolean enable, int source, int renderMode);

    native int enableAudioSource(long handle, boolean enable);

    native int setVolume(long handle, int volume);

    native long getDuration(long handle);

    native long getCurrentPlaybackTime(long handle);

    native int getCurrentVomume(long handle);

    native int getCurrentPlaybackState(long handle);

    native int getIndex(long handle);

}
